@@ -274,14 +274,21 @@ def split(url):
     l_query = rest.find('?')
     l_frag = rest.find('#')
     if l_path > 0:
-        netloc = rest[:l_path]
         if l_query > 0 and l_frag > 0:
+            netloc = rest[:l_path]
             path = rest[l_path:min(l_query, l_frag)]
         elif l_query > 0:
-            path = rest[l_path:l_query]
+            if l_query > l_path:
+                netloc = rest[:l_path]
+                path = rest[l_path:l_query]
+            else:
+                netloc = rest[:l_query]
+                path = ''
         elif l_frag > 0:
+            netloc = rest[:l_path]
             path = rest[l_path:l_frag]
         else:
+            netloc = rest[:l_path]
             path = rest[l_path:]
     else:
         if l_query > 0:
@@ -306,7 +313,12 @@ def split(url):
 def _clean_netloc(netloc):
     """Remove trailing '.' and ':' and tolower
     """
-    return netloc.rstrip('.:').decode('utf-8').lower().encode('utf-8')
+    try:
+        netloc.encode('ascii')
+    except:
+        return netloc.rstrip('.:').decode('utf-8').lower().encode('utf-8')
+    else:
+        return netloc.rstrip('.:').lower()
 
 
 def split_netloc(netloc):
@@ -343,6 +355,10 @@ def split_host(host):
         tld = '.'.join(parts[i:])
         wildcard_tld = '*.' + tld
         exception_tld = '!' + tld
+        if exception_tld in PSL:
+            domain = '.'.join(parts[:i+1])
+            tld = '.'.join(parts[i+1:])
+            break
         if tld in PSL:
             domain = '.'.join(parts[:i])
             break
@@ -350,10 +366,6 @@ def split_host(host):
             domain = '.'.join(parts[:i-1])
             tld = '.'.join(parts[i-1:])
             break
-        if exception_tld in PSL:
-            domain = '.'.join(parts[:i-1])
-            tld = '.'.join(parts[i-1:])
-            break
     if '.' in domain:
         (subdomain, domain) = domain.rsplit('.', 1) 
     return subdomain, domain, tld
